/*
 * SR1C1.c:
 * Control program for London Taxi Robot 1
 * compile with gcc R4C1.c -I/usr/local/include -L/usr/local/lib -lwiringPi -lm -o R4C1
 */

#include <wiringPi.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <math.h>
#include <termios.h>
#include <string.h>

#include <sys/ioctl.h>
#include <fcntl.h> 
#include <sys/stat.h> 
#include <sys/types.h> 
#include <unistd.h>  


#define COM_PULSE        0
#define COM_SEND         2
#define COM_RECEIVE      3

#define IR_SENSOR        5
#define IR_DIODE         1

#define MAX_LINE_SIZE 1000

//Delay is defined as 900, but considering the delay caused to stop a pulse, 700 seems to work better
//Even longer pulse length required for Raspberry module because of non realtime processing
#define PULSE_LENGTH 2700


static void usage(){
  printf("usage:\n");
  printf("R4C1 s (0-255)\n");
}


int main (int argc, char **argv){
  int i, i2;
  char charTemp[1000]="";
  int Verbose = 0;
  struct timeval structStartTime, structEndTime;
  unsigned long startTime, endTime;

  int fd;
  FILE *IRID;
  char IrFileName[]="IrModuleInRange.txt";
  char code[MAX_LINE_SIZE];
  char codeOld[MAX_LINE_SIZE];
  int readBytes;
  long pulseTime, spaceTime, pulseNumber;
  int lastBit = -1;
  int minPulseTime = 700;
  int maxPulseTime = 1100;
  int spaceNewCode = 3000;
  int codeLength = 0;
  int firstThreePulses[3];
  int countFirstThreePulses = 0;
  int moduleNumber = 0;
  int codeOK = 0;
  


  if(argc<3){
    usage();
    return(0);
  }
  
  if(argc>3){
    if(strcmp(argv[3], "v") == 0){
      Verbose = 1;
    }
    if(argc>4){
      if(atoi(argv[4]) > 1){
        Verbose = atoi(argv[4]);
      }
    }
  }
  
  if(Verbose == 1){
    printf ("R4 Control program\n") ;
    printf("argc=%d, argv[1]=%s, argv[2]=%s\n", argc, argv[1], argv[2]);
  }

  if (wiringPiSetup () == -1)
    exit (1) ;


  pinMode (COM_PULSE, OUTPUT);
  pinMode (COM_SEND, OUTPUT);
  pinMode (COM_RECEIVE, INPUT);

  pinMode (IR_SENSOR, INPUT);   //IR receiver
  pullUpDnControl (IR_SENSOR, PUD_OFF);
  
  //Only GPIO 1 is for PWM hardware
  //Frequency = 19.2MHz / pwmSetClock / pwmSetRange
  pinMode (IR_DIODE, PWM_OUTPUT);   //IR diode
  pwmSetMode (PWM_MODE_MS);
  pwmSetClock (5);  
  pwmSetRange (101);
 
 
  if(strcmp(argv[1], "s") == 0){//Send byte to Arduino
    if(atoi(argv[2]) < 0 || atoi(argv[2]) > 255){
      printf("%s invalid value! Send byte must be between 0 and 255!\n", argv[2]);
      usage();
      return(0);
    }
    if(Verbose == 1){
      printf("Sending byte: %s\n", argv[2]);
    }
    int sendByte = atoi(argv[2]);
    for(i=0; i<8; i++){
      if((sendByte) & (1<<(i))){
        digitalWrite(COM_SEND, 1);
        if(Verbose == 1){
          printf("Bit nr %d is 1\n", i);
        }
      }
      else{
        digitalWrite(COM_SEND, 0);
        if(Verbose == 1){
          printf("Bit nr %d is 0\n", i);
        }
      }
      digitalWrite(COM_PULSE, 1);
      usleep(10000);
      digitalWrite(COM_PULSE, 0);
      usleep(10000);
    }
    digitalWrite(COM_SEND, 0);
    digitalWrite(COM_PULSE, 0);    
  }
 
  return 0 ;
}













